@@ -59,6 +59,7 @@ enum
   PROP_SELECTED_FILES,
   PROP_UI_MANAGER,
   PROP_WIDGET,
+  N_PROPERTIES
 };
 
 
@@ -135,6 +136,8 @@ struct _ThunarLauncher
   ThunarFile             *current_directory;
   GList                  *selected_files;
 
+  guint                   launcher_idle_id;
+
   GtkIconFactory         *icon_factory;
   GtkActionGroup         *action_group;
   GtkUIManager           *ui_manager;
@@ -164,6 +167,7 @@ struct _ThunarLauncherPokeData
 {
   GList *files;
   GList *resolved_files;
+  guint  directories_in_tabs : 1;
 };
 
 
@@ -183,6 +187,10 @@ static GQuark thunar_launcher_handler_quark;
 
 
 
+static GParamSpec *launcher_props[N_PROPERTIES] = { NULL, };
+
+
+
 G_DEFINE_TYPE_WITH_CODE (ThunarLauncher, thunar_launcher, G_TYPE_OBJECT,
     G_IMPLEMENT_INTERFACE (THUNAR_TYPE_BROWSER, NULL)
     G_IMPLEMENT_INTERFACE (THUNAR_TYPE_NAVIGATOR, thunar_launcher_navigator_init)
@@ -194,6 +202,7 @@ static void
 thunar_launcher_class_init (ThunarLauncherClass *klass)
 {
   GObjectClass *gobject_class;
+  gpointer      g_iface;
 
   /* determine the "thunar-launcher-handler" quark */
   thunar_launcher_handler_quark = g_quark_from_static_string ("thunar-launcher-handler");
@@ -204,25 +213,36 @@ thunar_launcher_class_init (ThunarLauncherClass *klass)
   gobject_class->get_property = thunar_launcher_get_property;
   gobject_class->set_property = thunar_launcher_set_property;
 
-  /* Override ThunarNavigator's properties */
-  g_object_class_override_property (gobject_class, PROP_CURRENT_DIRECTORY, "current-directory");
-
-  /* Override ThunarComponent's properties */
-  g_object_class_override_property (gobject_class, PROP_SELECTED_FILES, "selected-files");
-  g_object_class_override_property (gobject_class, PROP_UI_MANAGER, "ui-manager");
-
   /**
    * ThunarLauncher:widget:
    *
    * The #GtkWidget with which this launcher is associated.
    **/
-  g_object_class_install_property (gobject_class,
-                                   PROP_WIDGET,
-                                   g_param_spec_object ("widget",
-                                                        "widget",
-                                                        "widget",
-                                                        GTK_TYPE_WIDGET,
-                                                        EXO_PARAM_READWRITE));
+  launcher_props[PROP_WIDGET] =
+      g_param_spec_object ("widget",
+                           "widget",
+                           "widget",
+                           GTK_TYPE_WIDGET,
+                           EXO_PARAM_READWRITE);
+
+  /* Override ThunarNavigator's properties */
+  g_iface = g_type_default_interface_peek (THUNAR_TYPE_NAVIGATOR);
+  launcher_props[PROP_CURRENT_DIRECTORY] =
+      g_param_spec_override ("current-directory",
+                             g_object_interface_find_property (g_iface, "current-directory"));
+
+  /* Override ThunarComponent's properties */
+  g_iface = g_type_default_interface_peek (THUNAR_TYPE_COMPONENT);
+  launcher_props[PROP_SELECTED_FILES] =
+      g_param_spec_override ("selected-files",
+                             g_object_interface_find_property (g_iface, "selected-files"));
+
+  launcher_props[PROP_UI_MANAGER] =
+      g_param_spec_override ("ui-manager",
+                             g_object_interface_find_property (g_iface, "ui-manager"));
+
+  /* install properties */
+  g_object_class_install_properties (gobject_class, N_PROPERTIES, launcher_props);
 }
 
 
@@ -305,6 +325,10 @@ thunar_launcher_finalize (GObject *object)
   if (G_UNLIKELY (launcher->sendto_idle_id != 0))
     g_source_remove (launcher->sendto_idle_id);
 
+  /* be sure to cancel the launcher idle source */
+  if (G_UNLIKELY (launcher->launcher_idle_id != 0))
+    g_source_remove (launcher->launcher_idle_id);
+
   /* drop our custom icon factory for the application/action icons */
   gtk_icon_factory_remove_default (launcher->icon_factory);
   g_object_unref (launcher->icon_factory);
@@ -414,7 +438,7 @@ thunar_launcher_set_current_directory (ThunarNavigator *navigator,
     g_object_ref (G_OBJECT (current_directory));
 
   /* notify listeners */
-  g_object_notify (G_OBJECT (launcher), "current-directory");
+  g_object_notify_by_pspec (G_OBJECT (launcher), launcher_props[PROP_CURRENT_DIRECTORY]);
 }
 
 
@@ -453,7 +477,7 @@ thunar_launcher_set_selected_files (ThunarComponent *component,
       thunar_launcher_update (launcher);
 
       /* notify listeners */
-      g_object_notify (G_OBJECT (launcher), "selected-files");
+      g_object_notify_by_pspec (G_OBJECT (launcher), launcher_props[PROP_SELECTED_FILES]);
     }
 }
 
@@ -519,7 +543,7 @@ thunar_launcher_set_ui_manager (ThunarComponent *component,
     }
 
   /* notify listeners */
-  g_object_notify (G_OBJECT (launcher), "ui-manager");
+  g_object_notify_by_pspec (G_OBJECT (launcher), launcher_props[PROP_UI_MANAGER]);
 }
 
 
@@ -729,28 +753,31 @@ thunar_launcher_open_windows (ThunarLauncher *launcher,
 
 
 
-static void
-thunar_launcher_update (ThunarLauncher *launcher)
+static gboolean
+thunar_launcher_update_idle (gpointer data)
 {
-  const gchar  *context_menu_path;
-  const gchar  *file_menu_path;
-  GtkAction    *action;
-  gboolean      default_is_open_with_other = FALSE;
-  GList        *applications;
-  GList        *actions;
-  GList        *lp;
-  gchar        *tooltip;
-  gchar        *label;
-  gchar        *name;
-  gint          n_directories = 0;
-  gint          n_executables = 0;
-  gint          n_regulars = 0;
-  gint          n_selected_files = 0;
-  gint          n;
+  ThunarLauncher *launcher = THUNAR_LAUNCHER (data);
+  const gchar    *context_menu_path;
+  const gchar    *file_menu_path;
+  GtkAction      *action;
+  gboolean        default_is_open_with_other = FALSE;
+  GList          *applications;
+  GList          *actions;
+  GList          *lp;
+  gchar          *tooltip;
+  gchar          *label;
+  gchar          *name;
+  gint            n_directories = 0;
+  gint            n_executables = 0;
+  gint            n_regulars = 0;
+  gint            n_selected_files = 0;
+  gint            n;
 
   /* verify that we're connected to an UI manager */
   if (G_UNLIKELY (launcher->ui_manager == NULL))
-    return;
+    return FALSE;
+
+  GDK_THREADS_ENTER ();
 
   /* drop the previous addons ui controls from the UI manager */
   if (G_LIKELY (launcher->ui_addons_merge_id != 0))
@@ -925,7 +952,7 @@ thunar_launcher_update (ThunarLauncher *launcher)
            * Unfortunately this is not supported by GIO directly */
 
           /* drop the default application from the list */
-          applications = g_list_remove (applications, applications->data);
+          applications = g_list_delete_link (applications, applications);
         }
       else if (G_UNLIKELY (n_selected_files == 1))
         {
@@ -1024,13 +1051,105 @@ thunar_launcher_update (ThunarLauncher *launcher)
 
       /* FIXME Add desktop actions here. Unfortunately they are not supported by
        * GIO, so we'll have to roll our own thing here */
+
+      /* schedule an update of the "Send To" menu */
+      if (G_LIKELY (launcher->sendto_idle_id == 0))
+        {
+          launcher->sendto_idle_id = g_idle_add_full (G_PRIORITY_LOW, thunar_launcher_sendto_idle,
+                                                      launcher, thunar_launcher_sendto_idle_destroy);
+        }
     }
 
-  /* schedule an update of the "Send To" menu */
-  if (G_LIKELY (launcher->sendto_idle_id == 0))
+  GDK_THREADS_LEAVE ();
+
+  return FALSE;
+}
+
+
+
+static void
+thunar_launcher_update_idle_destroy (gpointer data)
+{
+  THUNAR_LAUNCHER (data)->launcher_idle_id = 0;
+}
+
+
+
+static void
+thunar_launcher_update_check (ThunarLauncher *launcher,
+                              GtkWidget      *menu)
+{
+  _thunar_return_if_fail (THUNAR_IS_LAUNCHER (launcher));
+  _thunar_return_if_fail (menu == NULL || GTK_IS_MENU (menu));
+
+  /* check if the menu is in a dirty state */
+  if (launcher->launcher_idle_id != 0)
     {
-      launcher->sendto_idle_id = g_idle_add_full (G_PRIORITY_LOW, thunar_launcher_sendto_idle,
-                                                  launcher, thunar_launcher_sendto_idle_destroy);
+      /* stop the timeout */
+      g_source_remove (launcher->launcher_idle_id);
+
+      /* force an update */
+      thunar_launcher_update_idle (launcher);
+
+      /* ui update */
+      gtk_ui_manager_ensure_update (launcher->ui_manager);
+
+      /* make sure the menu is positioned correctly after the
+       * interface update */
+      if (menu != NULL)
+        gtk_menu_reposition (GTK_MENU (menu));
+    }
+}
+
+
+
+static void
+thunar_launcher_update (ThunarLauncher *launcher)
+{
+  GSList    *proxies, *lp;
+  GtkWidget *menu;
+  gboolean   instant_update;
+
+  _thunar_return_if_fail (THUNAR_IS_LAUNCHER (launcher));
+
+  proxies = gtk_action_get_proxies (launcher->action_open);
+  instant_update = (proxies == NULL);
+  for (lp = proxies; lp != NULL; lp = lp->next)
+    {
+      menu = gtk_widget_get_ancestor (lp->data, GTK_TYPE_MENU);
+      if (G_LIKELY (menu != NULL))
+        {
+          /* instant update if a menu is visible */
+          if (gtk_widget_get_visible (menu))
+            instant_update = TRUE;
+
+          /* watch menu changes */
+          g_signal_handlers_disconnect_by_func (G_OBJECT (menu), G_CALLBACK (thunar_launcher_update_check), launcher);
+          g_signal_connect_swapped (G_OBJECT (menu), "show", G_CALLBACK (thunar_launcher_update_check), launcher);
+        }
+    }
+
+  /* stop pending timeouts */
+  if (launcher->launcher_idle_id != 0)
+    g_source_remove (launcher->launcher_idle_id);
+
+  if (instant_update)
+    {
+      /* directly update without interruption */
+      thunar_launcher_update_idle (launcher);
+    }
+  else
+    {
+      /* assume all actions are working */
+      gtk_action_set_sensitive (launcher->action_open, TRUE);
+      gtk_action_set_visible (launcher->action_open_with_other, TRUE);
+      gtk_action_set_visible (launcher->action_open_in_new_window, TRUE);
+      gtk_action_set_visible (launcher->action_open_in_new_tab, TRUE);
+      gtk_action_set_visible (launcher->action_open_with_other_in_menu, TRUE);
+
+      /* delayed update */
+      launcher->launcher_idle_id = g_timeout_add_seconds_full (G_PRIORITY_LOW, 5, thunar_launcher_update_idle,
+                                                               launcher, thunar_launcher_update_idle_destroy);
     }
 }
 
@@ -1169,8 +1288,17 @@ thunar_launcher_poke_files_finish (ThunarBrowser *browser,
       /* check if we have any directories to process */
       if (G_LIKELY (directories != NULL))
         {
-          /* open new windows for all directories */
-          thunar_launcher_open_windows (THUNAR_LAUNCHER (browser), directories);
+          if (poke_data->directories_in_tabs)
+            {
+              /* open new tabs */
+              for (lp = directories; lp != NULL; lp = lp->next)
+                thunar_navigator_open_new_tab (THUNAR_NAVIGATOR (browser), lp->data);
+            }
+          else
+            {
+              /* open new windows for all directories */
+              thunar_launcher_open_windows (THUNAR_LAUNCHER (browser), directories);
+            }
           g_list_free (directories);
         }
 
@@ -1216,6 +1344,11 @@ thunar_launcher_action_open (GtkAction      *action,
   _thunar_return_if_fail (GTK_IS_ACTION (action));
   _thunar_return_if_fail (THUNAR_IS_LAUNCHER (launcher));
 
+  /* force update if still dirty */
+  thunar_launcher_update_check (launcher, NULL);
+  if (!gtk_action_get_sensitive (action))
+    return;
+
   /* check if we have a mime handler associated with the action */
   app_info = g_object_get_qdata (G_OBJECT (action), thunar_launcher_handler_quark);
   if (G_LIKELY (app_info != NULL))
@@ -1225,19 +1358,22 @@ thunar_launcher_action_open (GtkAction      *action,
       thunar_launcher_open_paths (app_info, selected_paths, launcher);
       thunar_g_file_list_free (selected_paths);
     }
-  else if (g_list_length (launcher->selected_files) == 1)
+  else if (launcher->selected_files != NULL)
     {
-      thunar_browser_poke_file (THUNAR_BROWSER (launcher),
-                                launcher->selected_files->data, launcher->widget,
-                                thunar_launcher_poke_file_finish, NULL);
-    }
-  else
-    {
-      /* resolve files one after another until none is left. Open/execute
-       * the resolved files/directories when all this is done at a later
-       * stage */
-      poke_data = thunar_launcher_poke_data_new (launcher->selected_files);
-      thunar_launcher_poke_files (launcher, poke_data);
+      if (launcher->selected_files->next == NULL)
+        {
+          thunar_browser_poke_file (THUNAR_BROWSER (launcher),
+                                    launcher->selected_files->data, launcher->widget,
+                                    thunar_launcher_poke_file_finish, NULL);
+        }
+      else
+        {
+          /* resolve files one after another until none is left. Open/execute
+           * the resolved files/directories when all this is done at a later
+           * stage */
+           poke_data = thunar_launcher_poke_data_new (launcher->selected_files);
+           thunar_launcher_poke_files (launcher, poke_data);
+        }
     }
 }
 
@@ -1250,6 +1386,11 @@ thunar_launcher_action_open_with_other (GtkAction      *action,
   _thunar_return_if_fail (GTK_IS_ACTION (action));
   _thunar_return_if_fail (THUNAR_IS_LAUNCHER (launcher));
 
+  /* force update if still dirty */
+  thunar_launcher_update_check (launcher, NULL);
+  if (!gtk_action_get_visible (action))
+    return;
+
   /* verify that we have atleast one selected file */
   if (G_LIKELY (launcher->selected_files != NULL))
     {
@@ -1264,11 +1405,19 @@ static void
 thunar_launcher_action_open_in_new_window (GtkAction      *action,
                                            ThunarLauncher *launcher)
 {
+  ThunarLauncherPokeData *poke_data;
+
   _thunar_return_if_fail (GTK_IS_ACTION (action));
   _thunar_return_if_fail (THUNAR_IS_LAUNCHER (launcher));
 
+  /* force update if still dirty */
+  thunar_launcher_update_check (launcher, NULL);
+  if (!gtk_action_get_visible (action))
+    return;
+
   /* open the selected directories in new windows */
-  thunar_launcher_open_windows (launcher, launcher->selected_files);
+  poke_data = thunar_launcher_poke_data_new (launcher->selected_files);
+  thunar_launcher_poke_files (launcher, poke_data);
 }
 
 
@@ -1277,21 +1426,20 @@ static void
 thunar_launcher_action_open_in_new_tab (GtkAction      *action,
                                         ThunarLauncher *launcher)
 {
-  GList *lp;
-  GList *selected_files;
+  ThunarLauncherPokeData *poke_data;
 
   _thunar_return_if_fail (GTK_IS_ACTION (action));
   _thunar_return_if_fail (THUNAR_IS_LAUNCHER (launcher));
 
+  /* force update if still dirty */
+  thunar_launcher_update_check (launcher, NULL);
+  if (!gtk_action_get_visible (action))
+    return;
+
   /* open all selected directories in a new tab */
-  selected_files = thunar_g_file_list_copy (launcher->selected_files);
-  for (lp = selected_files; lp != NULL; lp = lp->next)
-    {
-      if (thunar_file_is_directory (lp->data))
-        thunar_navigator_open_new_tab (THUNAR_NAVIGATOR (launcher), lp->data);
-      g_object_unref (G_OBJECT (lp->data));
-    }
-  g_list_free (selected_files);
+  poke_data = thunar_launcher_poke_data_new (launcher->selected_files);
+  poke_data->directories_in_tabs = TRUE;
+  thunar_launcher_poke_files (launcher, poke_data);
 }
 
 
@@ -1365,6 +1513,7 @@ thunar_launcher_poke_data_new (GList *files)
   data = g_slice_new0 (ThunarLauncherPokeData);
   data->files = thunar_g_file_list_copy (files);
   data->resolved_files = NULL;
+  data->directories_in_tabs = FALSE;
 
   return data;
 }
@@ -1447,7 +1596,6 @@ thunar_launcher_action_sendto_device (GtkAction      *action,
 {
   ThunarLauncherMountData *data;
   GMountOperation         *mount_operation;
-  GtkWidget               *window;
   ThunarDevice            *device;
   GList                   *files;
 
@@ -1467,14 +1615,11 @@ thunar_launcher_action_sendto_device (GtkAction      *action,
   /* make sure to mount the device first, if it's not already mounted */
   if (!thunar_device_is_mounted (device))
     {
-      /* determine the toplevel window */
-      window = gtk_widget_get_toplevel (launcher->widget);
-
       /* allocate mount data */
       data = thunar_launcher_mount_data_new (launcher, files);
 
       /* allocate a GTK+ mount operation */
-      mount_operation = gtk_mount_operation_new (GTK_WINDOW (window));
+      mount_operation = thunar_gtk_mount_operation_new (launcher->widget);
 
       /* try to mount the device and later start sending the files */
       thunar_device_mount (device,
@@ -1741,7 +1886,7 @@ thunar_launcher_set_widget (ThunarLauncher *launcher,
     }
 
   /* notify listeners */
-  g_object_notify (G_OBJECT (launcher), "widget");
+  g_object_notify_by_pspec (G_OBJECT (launcher), launcher_props[PROP_WIDGET]);
 }
 
 
